/*  This file is part of distributed Structured Prediction (dSP) - http://www.alexander-schwing.de/
*
*  distributed Structured Prediction (dSP) is free software: you can
*  redistribute it and/or modify it under the terms of the GNU General
*  Public License as published by the Free Software Foundation, either
*  version 3 of the License, or (at your option) any later version.
*
*  distributed Structured Prediction (dSP) is distributed in the hope
*  that it will be useful, but WITHOUT ANY WARRANTY; without even the
*  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
*  PURPOSE. See the GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with distributed Structured Prediction (dSP).
*  If not, see <http://www.gnu.org/licenses/>.
*
*  Copyright (C) 2010-2013  Alexander G. Schwing  [http://www.alexander-schwing.de/]
*/

//Author: Alexander G. Schwing

/*  ON A PERSONAL NOTE: I spent a significant amount of time to go through
*  both, theoretical justifications and coding of this framework.
*  I hope the package is useful for your task. Any requests,
*  feedback, donations and support would be greatly appreciated.
*  Thank you for contacting me!
*/

#ifndef __lib_BP__
#define __lib_BP__

#include "../Settings.h"

#include <vector>
#include <map>
#include <set>
#include <stddef.h>

#include "../libCRF/libCRF.h"

template <class T>
class libBP {
private:
	size_t num_nodes;
	size_t num_factors;
	T epsilon;

	std::map<size_t, std::map<size_t, std::vector<T> > > log_n;

	std::map<size_t, std::pair<struct PotentialStruct<T>, std::set<size_t> > > *LocalPot;
	std::map<size_t, std::pair<struct PotentialStruct<T>, std::set<size_t> > > *FactorPot;
	
	void computeLocalBelief(std::map<size_t, std::vector<T> > &localBelief);
	void computeN();
	T computeM(size_t v, size_t alpha, size_t x_i, size_t v_num_states);
	T computeDual();
	T computePrimal();
public:
	libBP();
	~libBP();

	int Initialize(std::map<size_t, std::pair<struct PotentialStruct<T>, std::set< size_t > > > &LocalPot, std::map<size_t, std::pair<struct PotentialStruct<T>, std::set< size_t > > > &FactorPot, T epsilon = T(1.0));
	int RunBP(int MaxIter, bool debug = false);
	int GetResult(std::map<size_t, std::vector<T> > &localBelief);
	T GetEntropySum();
};

#endif